package me.unurled.sacredrealms.sr.components.player;

import static me.unurled.sacredrealms.sr.utils.Logger.warn;

import java.util.UUID;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import org.bukkit.Bukkit;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SRPlayer {

  private final UUID uuid;

  public SRPlayer(@NotNull Player player) {
    this.uuid = player.getUniqueId();
  }

  @NotNull
  public UUID getUuid() {
    return uuid;
  }

  @Nullable
  public Object getAttribute(@NotNull Attribute attribute) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return null;
    }
    PersistentDataContainer pdc = p.getPersistentDataContainer();
    return pdc.get(new NamespacedKey("sr", attribute.getID()), attribute.getType());
  }

  public void setAttribute(@NotNull Attribute attribute, @NotNull Object value) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return;
    }
    PersistentDataContainer pdc = p.getPersistentDataContainer();
    if (!attribute.getType().getComplexType().isInstance(value)) {
      return;
    }
    if (attribute.getType().equals(PersistentDataType.DOUBLE)) {
      double d = (double) value;
      if (d < attribute.getMinValue() || d > attribute.getMaxValue()) {
        warn(
            "(Player "
                + p.getName()
                + ") Value "
                + d
                + " for attribute "
                + attribute.getID()
                + " is out of bounds. Min: "
                + attribute.getMinValue()
                + " Max: "
                + attribute.getMaxValue()
                + " Value: "
                + d);
        return;
      }
    } else if (attribute.getType().equals(PersistentDataType.INTEGER)) {
      int i = (int) value;
      if (i < attribute.getMinValue() || i > attribute.getMaxValue()) {
        warn(
            "(Player "
                + p.getName()
                + ") Value "
                + i
                + " for attribute "
                + attribute.getID()
                + " is out of bounds. Min: "
                + attribute.getMinValue()
                + " Max: "
                + attribute.getMaxValue()
                + " Value: "
                + i);

        return;
      }
    }
    pdc.set(new NamespacedKey("sr", attribute.getID()), attribute.getType(), value);
  }

  @Nullable
  public Object getHandItemAttribute(@NotNull Attribute attribute) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return null;
    }
    return p.getInventory()
        .getItemInMainHand()
        .getItemMeta()
        .getPersistentDataContainer()
        .get(new NamespacedKey("sr", attribute.getID()), attribute.getType());
  }

  public void setHandItemAttribute(@NotNull Attribute attribute, @NotNull Object value) {
    Player p = Bukkit.getPlayer(uuid);
    if (p == null) {
      return;
    }
    if (!attribute.getType().getComplexType().isInstance(value)) {
      return;
    }
    if (attribute.getType().equals(PersistentDataType.DOUBLE)) {
      double d = (double) value;
      if (d < attribute.getMinValue() || d > attribute.getMaxValue()) {
        warn(
            "(Player "
                + p.getName()
                + ") Value "
                + d
                + " for attribute "
                + attribute.getID()
                + " is out of bounds. Min: "
                + attribute.getMinValue()
                + " Max: "
                + attribute.getMaxValue()
                + " Value: "
                + d);
        return;
      }
    } else if (attribute.getType().equals(PersistentDataType.INTEGER)) {
      int i = (int) value;
      if (i < attribute.getMinValue() || i > attribute.getMaxValue()) {
        warn(
            "(Player "
                + p.getName()
                + ") Value "
                + i
                + " for attribute "
                + attribute.getID()
                + " is out of bounds. Min: "
                + attribute.getMinValue()
                + " Max: "
                + attribute.getMaxValue()
                + " Value: "
                + i);

        return;
      }
    }
    ItemStack i = p.getInventory().getItemInMainHand();
    ItemMeta im = i.getItemMeta();
    im.getPersistentDataContainer()
        .set(new NamespacedKey("sr", attribute.getID()), attribute.getType(), value);
    im.lore();
  }
}
